/* 'new' config.h:
 *	A configuration file designed to make best use of the abilities
 *	of ircII, and trying to make things more intuitively understandable.
 *
 * Original: Michael Sandrof
 * V2 by Carl V. Loesch (lynx@dm.unirm1.it)
 * V2.EPIC by jfn (nelson@cs.uwp.edu)
 */

#ifndef _CONFIG_H_
#define _CONFIG_H_

/* 
 *  This is where you should set your default server list.  The list should
 *  be a space seperated list of the form hostname:portnum:password.  The
 *  portnum and password are optional.  An example is:
 *
 *	#define DEFAULT_SERVER "irc.iastate.edu irc-2.mit.edu:6666:lag-2sux"
 *
 *  If you choose to #define SERVER_FILES, this #define will be ignored unless
 *  the SERVER_FILES is empty or cannot be opened.
 *
 *  The client will not operate if you do not provide a server to use.
 */
#define DEFAULT_SERVER "/tmp/.ircd/6667 irc-2.mit.edu"


/*
 *  Bowing to popular demand, i have tentatively started snipping away
 *  at the non-essential features of ircII.  If you #define MICRO_EPIC,
 *  then several "optional" features of ircII will not be available to
 *  you.  However, the client supposedly should be much smaller and
 *  maybe even a little faster.
 *
 *  Current list of features turned off:
 *   * /ENCRYPTion
 *   * /ALIASes and built-in functions.
 *   * /ONs
 *   * /EXEC
 *
 */
#undef MICRO_EPIC

/*
 *  ircII has a security feature for paranoid sysadmins or for those
 *  users whose sysadmins are paranoid.  The security feature allows
 *  you to restrict who may use the ircII client.  You have four options:
 *
 *	1) compile into the binary a list of uids who can use the program
 *		*Pros: cant be hacked -- very secure
 *		*Cons: cant be changed w/o recompiling
 *	2) compile into the binary a file which will contain the uids of
 *	   the people who can use the program
 *		*Pros: can be changed as you need by just editing the fil
 *		*Cons: since the uids are in a file, prone to hacking
 *	3) compile into the binary a file which will contain the uids of
 *	   the people who cannot use the program
 *		*Pros: allows for public use and allow you to exclude
 *		       troublemakers without enumerating everyone else
 *		*Cons: since the uids are in a file, prone to hacking
 *	4) compile into the binary a password
 *		*Pros: cant be hacked -- secure
 *		*Cons: cant be changed w/o recompiling
 *
 *    The first two options are mutually exclusive.  The third and fourth
 *    options can be specified at your option.  If you specify both the
 *    first and second options, the first option has precedence.
 */

/*
 *   To use the first security feature, #define HARD_SECURE.  You will also
 *   have to #define VALID_UIDS which will be a list of those uids (integers,
 *   not usernames) which will be allowed to execute the resulting program.
 *   If you #define HARD_SECURE but do not define #VALID_UIDS, then noone
 *   will be able to execute the program!
 */
#undef HARD_SECURE
#define VALID_UIDS "100 101"

/*
 *  To use the second security measure, simply #define SOFT_SECURE to a 
 *  filename that will be world-readable that will contain the uids of
 *  all the users who will be allowed to execute the program.  It is important
 *  that this file be readable by at least every person who can execute the
 *  program or this security measure will be comprimised.
 *
 *  The uid file should have one uid per line (integer, not username).
 *
 *  You can define VALID_UID_FILE, but if SOFT_SECURE is not defined, it will
 *  not be used.
 */
#undef SOFT_SECURE
#define VALID_UID_FILE "/home/jnelson/..."

/*
 *  This allows you to use the third security option.  If you define this,
 *  it should be assigned to a file that will contain a listing of all of
 *  the uids (integers, not usernames) that will not be allowed to execute
 *  the resulting program.
 */
/*#define INVALID_UID_FILE "/home/jnelson/...."*/


/*
 *  This allows you to use the fourth security option.  If you define this,
 *  the program will prompt the user to enter this prompt before it will
 *  continue executing the program.  This password does not affect in any
 *  way the other protection schemes.  A user who is not allowed to run
 *  the program will not be allowed to use the program even if they know
 *  the password.
 */
/*#define PASSWORD "booya"*/

/*
 * This is the fun part.  If someone runs your program who shouldnt run
 * it, either because the dont know the password or because they arent
 * on the valid list or whatever, ircII will execute this program to 
 * "spoof" them into thinking your program is actually some other program.
 *
 * This can be defined to any valid C expression that will resolve to a
 * character string. (ie, a character literal or function call)
 */
#define SPOOF_PROGRAM "getenv(\"SHELL\")"

/*
 * Define this if you want compatability with the old ircd2.7 servers.
 * If you do not ever use ircd2.7 servers, you should probably leave this
 * undefined, as you will save some space in your binary.
 */
#define COMPAT_27

/*
 * Set the following to 1 if you wish for IRCII not to disturb the tty's flow
 * control characters as the default.  Normally, these are ^Q and ^S.  You
 * may have to rebind them in IRCII.  Set it to 0 for IRCII to take over the
 * tty's flow control.
 */
#define USE_FLOW_CONTROL 1


/* This should be defined to the thing that is used in your password files
 * to seperate the fields of the user information.  Dont change this unless
 * you know for sure that your password file doesnt use the comma. (most do)
 */
#define GECOS_DELIMITER ',' 

/*
 * Below you can set what type of mail your system uses and the path to the
 * appropriate mail file/directory.  Only one may be selected. 
 * You may also undefine both if you think mail checking in irc clients is
 * silly.
 *
 * this should probably be grokked by configure some day....
 */
/* AMS_MAIL is the Andrew Mail System mail format. */
#undef AMS_MAIL

/* UNIX_MAIL is the normal unix mail format.  */
#undef UNIX_MAIL "/usr/spool/mail"

#ifdef AMS_MAIL
# define AMS_MAIL "Mailbox"
#endif /* AMS_MAIL */

/*
 * MAIL_DELIMITER specifies the unique text that separates one mail message
 * from another in the mail spool file when using UNIX_MAIL.
 */
#define MAIL_DELIMITER "From "


/*
 * ALLOC_DEBUG forces ircII to keep some statistics on malloc/free.
 * If this doesnt mean anything to you, then leave this #undef'd
 *
 * OBWARNING: This will probably cause you coredumps if you have a
 * 	      large .ircrc.  I dont know of any significant memory
 *	      leaks in ircII, however.
 */
#undef ALLOC_DEBUG

/* If you define UNAME_HACK, the uname information displayed in the
 * CTCP VERSION info will appear as "*IX" irregardless of any other
 * settings.  Useful for paranoid users who dont want others to know
 * that theyre running a buggy SunOS machine. >;-)
 */
#undef UNAME_HACK


/* If defined, COMMENT_HACK will only allow C-like comments to appear
 * at the beginning of a line.  (To be precise, the opening "slash-star"
 * must be at the beginning of a line, and the closing "star-slash" may be
 * anywhere. This increases compatability with older ircII scripts that
 * dont know about the "slash-star star-slash" comments (like PhoEnIx)
 */
/* Any script that defines an alias '*' and then calls it with "slash-star"
 * loses.  No aplogies.
 */
#define COMMENT_HACK


/* If defined, you are able to suspend the irc process by pressing ^Z.
 * if undef'd, you are not able to suspend the irc process.
 */
#define ALLOW_STOP_IRC


/* And here is the port number for default client connections.  */
#define IRC_PORT 6667

/*
 * For those of you who dont want to use the disk space for help files,
 * there are a few automatic help clients out there to service you.  
 * Just define DEFAULT_HELP_SERVICE to the automatic client nearest you.
 *
 *  NOTE: it is important to leave the double quotes (") in here.
 *
 *	"IrcIIhelp"
 *	"help_AU"
 *	"help_US"
 *	"help_EU"
 *	"help_UK"
 */
#define	DEFAULT_HELP_SERVICE "help_US"


/*
 * If you want to have a file containing the list of irc servers to 
 * use, define SERVERS_FILE to be that filename.  Put the file in the 
 * ircII library directory.  This file should be whitespace seperated
 * hostname:portnum:password (with the portnum and password being
 * optional).  This server list will supercede the DEFAULT_SERVER
 */
/*#define SERVERS_FILE "ircII.servers"*/


/*
 * Message Of The Day file
 *
 * There has been a lot of confusion surrounding this setting. 
 * I will attempt to explain how this works, so that when it doesnt
 * work in the way you are expecting, you wont gripe at me for it
 * being a bug. :P
 *
 * 1) This displays the motd <*>ONCE<*> per time you change it.  Once
 * they see it, they never see it again until you change the MOTD.
 *
 * 2) If you want them to see it every time, you have to #define
 * ALWAYS_SHOW_MOTD which was an "undocumented" feature until now.
 *
 * 3) the MOTD file must be in the TOP level of the irclib directory.
 * This is the same directory as the "script" "translation" and "help"
 * directories!
 *
 * I still think a better alternative is to cat the file using the
 * the file operators and putting it in local.  Maybe ill write a
 * script to do this.
 */
#undef MOTD_FILE "ircII.motd"
#undef PAUSE_AFTER_MOTD 1
#define ALWAYS_SHOW_MOTD

/*
 * define this if you want your irc client to exit after an 
 * operator kill.
 */
#undef QUIT_ON_OPERATOR_KILL


/*
 * Below are the IRCII variable defaults.  For boolean variables, use 1 for
 * ON and 0 for OFF.  You may set string variable to NULL if you wish them to
 * have no value.  None of these are optional.  You may *not* comment out or
 * remove them.  They are default values for variables and are required for
 * proper compilation.
 */
#define DEFAULT_ALWAYS_SPLIT_BIGGEST 1
#define DEFAULT_AUTO_NEW_NICK 1
#define DEFAULT_AUTO_RECONNECT 1
#define DEFAULT_AUTO_REJOIN 1
#define DEFAULT_AUTO_UNMARK_AWAY 0
#define DEFAULT_AUTO_WHOWAS 1
#define DEFAULT_BEEP 1
#define DEFAULT_BEEP_MAX 3
#define DEFAULT_BEEP_ON_MSG "NONE"
#define DEFAULT_BEEP_WHEN_AWAY 1
#define	DEFAULT_BOLD_VIDEO 1
#define DEFAULT_CHANNEL_NAME_WIDTH 0
#define DEFAULT_CLOCK 1
#define DEFAULT_CLOCK_24HOUR 0
#define DEFAULT_CLOCK_ALARM NULL
#define DEFAULT_CMDCHARS "/"
#define DEFAULT_COMMAND_COMPLETION 1
#define DEFAULT_COMMAND_MODE 0
#define DEFAULT_CONTINUED_LINE "+"
#define DEFAULT_DISPLAY 1
#define DEFAULT_EIGHT_BIT_CHARACTERS 0
#define DEFAULT_ENCRYPT_PROGRAM NULL
#define DEFAULT_EXEC_PROTECTION 0
#define DEFAULT_FLOATING_POINT_MATH 0
#define DEFAULT_FLOOD_AFTER 3
#define DEFAULT_FLOOD_RATE 3
#define DEFAULT_FLOOD_USERS 3
#define DEFAULT_FLOOD_WARNING 0
#define DEFAULT_FULL_STATUS_LINE 1
#define DEFAULT_HELP_PAGER 1
#define DEFAULT_HELP_PROMPT 1
#define DEFAULT_HELP_WINDOW 0
#define DEFAULT_HIDE_PRIVATE_CHANNELS 0
#define DEFAULT_HIGHLIGHT_CHAR "BOLD"
#define DEFAULT_HISTORY 150
#define DEFAULT_HISTORY_FILE NULL
#define DEFAULT_HOLD_MODE 0
#define DEFAULT_HOLD_MODE_MAX 0
#define DEFAULT_INDENT 0
#define DEFAULT_INPUT_ALIASES 0
#define DEFAULT_INPUT_PROMPT "> "
#define DEFAULT_INPUT_PROTECTION 0
#define DEFAULT_INSERT_MODE 1
#define DEFAULT_INVERSE_VIDEO 1
#define DEFAULT_LASTLOG 1000
#define DEFAULT_LASTLOG_LEVEL "ALL"
#define DEFAULT_LOG 0
#define DEFAULT_LOGFILE "irc.log"
#define DEFAULT_MAIL 2
#define DEFAULT_MAX_RECURSIONS 25
#define DEFAULT_MINIMUM_SERVERS 0
#define DEFAULT_MINIMUM_USERS 0
#define DEFAULT_MODE_STRIPPER 1
#define DEFAULT_NO_CTCP_FLOOD 1
#define DEFAULT_NOTIFY_HANDLER "NOISY"
#define DEFAULT_NOTIFY_LEVEL "ALL"
#define DEFAULT_NOTIFY_ON_TERMINATION 1
#define DEFAULT_NUM_OF_WHOWAS 1
#define DEFAULT_SCROLL 1
#define DEFAULT_SCROLL_LINES 1
#define DEFAULT_SECURITY 0
#define DEFAULT_SEND_IGNORE_MSG 0
#define DEFAULT_SHELL "/bin/sh"
#define DEFAULT_SHELL_FLAGS "-c"
#define DEFAULT_SHELL_LIMIT 0
#define DEFAULT_SHOW_AWAY_ONCE 1
#define DEFAULT_SHOW_CHANNEL_NAMES 1
#define DEFAULT_SHOW_END_OF_MSGS 1
#define DEFAULT_SHOW_NUMERICS 0
#define	DEFAULT_SHOW_STATUS_ALL 0
#define DEFAULT_SHOW_WHO_HOPCOUNT 1
#define DEFAULT_STATUS_AWAY " (Away)"
#define DEFAULT_STATUS_CHANNEL " %C"
#define DEFAULT_STATUS_CHANOP "@"
#define DEFAULT_STATUS_CLOCK " %T"
#define DEFAULT_STATUS_FORMAT "%T [%R] %*%@%N%#%S%H%B%Q%A%C%+%I%O%M%F %D %U %W"
#define DEFAULT_STATUS_HOLD "Held: "
#define DEFAULT_STATUS_HOLD_LINES "%B"
#define DEFAULT_STATUS_INSERT ""
#define DEFAULT_STATUS_MODE " (+%+)"
#define DEFAULT_STATUS_MAIL " (M: %M)"
#define DEFAULT_STATUS_NO_REPEAT 0
#define	DEFAULT_STATUS_NOTIFY " (W: %F)"
#define DEFAULT_STATUS_OPER "*"
#define DEFAULT_STATUS_OVERWRITE " (Overwrite)"
#define DEFAULT_STATUS_QUERY " (Query: %Q)"
#define DEFAULT_STATUS_SERVER " (%S)"
#define DEFAULT_STATUS_UMODE " (+%#)"
#define DEFAULT_STATUS_USER "ircII2.8.2-EPIC3 -- Type /help for help"
#define DEFAULT_STATUS_USER1 ""
#define DEFAULT_STATUS_USER2 ""
#define DEFAULT_STATUS_USER3 ""
/*
 * You must have SHOW_STATUS_ALL set to 0 for the following STATUS_USERs to show
 * These only appear in your non-current window	-- Jake [WinterHawk] Khuon
 */
#define DEFAULT_STATUS_USER4 ""
#define DEFAULT_STATUS_USER5 ""
#define DEFAULT_STATUS_USER6 ""
#define DEFAULT_STATUS_USER7 ""
#define DEFAULT_STATUS_USER8 ""
#define DEFAULT_STATUS_USER9 ""
#define DEFAULT_STATUS_WINDOW "^^^^^^^^"
#define DEFAULT_SUPPRESS_SERVER_MOTD 0
#define DEFAULT_SUPPRESS_FROM_REMOTE_SERVER 0
#define DEFAULT_TAB 1
#define	DEFAULT_TAB_MAX 0
#define DEFAULT_UNDERLINE_VIDEO 1
#define DEFAULT_USE_OLD_MSG 0
#define DEFAULT_USERINFO "irc2.8.2-EPIC3, only the best!"
#define	DEFAULT_USER_WALLOPS 1
#define DEFAULT_VERBOSE_CTCP 1
#define DEFAULT_WARN_OF_IGNORES 0
#define DEFAULT_XTERM_OPTIONS NULL

/*
 * IF YOU CAME HERE LOOKING FOR 
 *
 *	#define COMMAND_LINE_B
 *	#define COMMAND_LINE_L
 * 	#define ON_KICK
 *
 * DONT BOTHER!  Because they arent checked any more.  There is no way
 * to turn off the -b, -l flags, or the /on kick feature.  They are on
 * permanently.  If you want to remove them, youll have to do it yourself.
 */
/* define these if you want them included */
#undef	DEBUG				/* Just what it says... */
#undef  ENFORCE_STRICTER_PROTOCOL	/* Disallow things that are not */
					/* technically illegal */
#undef	I_AM_A_FASCIST_BASTARD		/* Enforce things that shouldnt be */
					/* illegal, but included for paranoid
					   sysadmins and other asses. */
#define	EXEC_COMMAND			/* Define if you want the /exec */
					/* command.  If you dont want it,
					   #undef this. Pretty easy, eh? */

/* Dont change these -- theyre important. */
#if defined(VALID_UIDS) && !defined(HARD_SECURE)
#undef VALID_UIDS
#endif

#if defined(PASSWORD) && !defined(HARD_SECURE) && !defined(SOFT_SECURE)
#undef PASSWORD
#endif

#if defined(HARD_SECURE) && !defined(VALID_UIDS)
You must #define VALID_UIDS if you #define HARD_SECURE
#endif

#if defined(VALID_UID_FILE) && !defined(SOFT_SECURE)
#undef VALID_UID_FILE
#endif

#if defined(SOFT_SECURE) && !defined(VALID_UID_FILE)
You must #define VALID_UID_FILE if you #define SOFT_SECURE
#endif

#ifndef SPOOF_PROGRAM
#define SPOOF_PROGRAM "/bin/sh"
#endif
/* end of section not to change */

#endif /* _CONFIG_H_ */

